/**
 * @typedef {import('socket.io').Namespace} Namespace
 * @typedef {import('socket.io').Socket} Socket
 * @typedef {import('socket.io').Server} Server
 *
 * @typedef {import('../subscriber.js').Subscriber} Subscriber
 */
/**
 * @typedef UserLike
 * @prop {string} userid
 *
 * @typedef {'sync-step-1' | 'sync-step-2' | 'sync-update' | 'awareness-update'} EventType
 *
 * @typedef {{
 *   ydoc: Y.Doc;
 *   awareness: AwarenessProtocol.Awareness;
 *   redisLastId: string;
 *   storeReferences: any[] | null;
 * }} RedisDoc
 */
/**
 * @typedef YSocketIOConfiguration
 * YSocketIO instance configuration. Here you can configure:
 * - authenticate: The callback to authenticate the client connection
 *
 * @prop {(socket: Socket)=> Promise<UserLike | null> | UserLike | null} authenticate
 * Callback to authenticate the client connection.
 */
/**
 * YSocketIO class. This handles document synchronization.
 */
export class YSocketIO {
    /**
     * YSocketIO constructor.
     * @constructor
     * @param {Server} io Server instance from Socket IO
     * @param {YSocketIOConfiguration} configuration The YSocketIO configuration
     */
    constructor(io: Server, configuration: YSocketIOConfiguration);
    /**
     * @type {Server}
     * @private
     * @readonly
     */
    private readonly io;
    /**
     * @type {YSocketIOConfiguration}
     * @private
     * @readonly
     */
    private readonly configuration;
    /**
     * @type {Namespace | null}
     * @public
     */
    public nsp: Namespace | null;
    /**
     * @type {api.Api | null}
     * @private
     */
    private client;
    /**
     * @type {Subscriber | null}
     * @private
     */
    private subscriber;
    /**
     * @type {Map<string, string>}
     * @private
     * @readonly
     */
    private readonly namespaceStreamMap;
    /**
     * @type {Map<string, string>}
     * @private
     * @readonly
     */
    private readonly streamNamespaceMap;
    /**
     * @type {Map<string, Namespace>}
     * @private
     * @readonly
     */
    private readonly namespaceMap;
    /**
     * YSocketIO initialization.
     *
     *  This method set ups a dynamic namespace manager for namespaces that match with the regular expression `/^\/yjs\|.*$/`
     *  and adds the connection authentication middleware to the dynamics namespaces.
     *
     *  It also starts socket connection listeners.
     * @param {import('../storage.js').AbstractStorage} store
     * @param {{ redisPrefix?: string }=} redisPrefix
     * @public
     */
    public initialize(store: import('../storage.js').AbstractStorage, { redisPrefix }?: {
        redisPrefix?: string;
    } | undefined): Promise<{
        client: api.Api;
        subscriber: import("../subscriber.js").Subscriber;
    }>;
    /**
     * This function initializes the socket event listeners to synchronize document changes.
     *
     *  The synchronization protocol is as follows:
     *  - A client emits the sync step one event (`sync-step-1`) which sends the document as a state vector
     *    and the sync step two callback as an acknowledgment according to the socket io acknowledgments.
     *  - When the server receives the `sync-step-1` event, it executes the `syncStep2` acknowledgment callback and sends
     *    the difference between the received state vector and the local document (this difference is called an update).
     *  - The second step of the sync is to apply the update sent in the `syncStep2` callback parameters from the server
     *    to the document on the client side.
     *  - There is another event (`sync-update`) that is emitted from the client, which sends an update for the document,
     *    and when the server receives this event, it applies the received update to the local document.
     *  - When an update is applied to a document, it will fire the document's "update" event, which
     *    sends the update to clients connected to the document's namespace.
     * @private
     * @param {Socket} socket The socket connection
     * @readonly
     */
    private readonly initSyncListeners;
    /**
     * This function initializes socket event listeners to synchronize awareness changes.
     *
     *  The awareness protocol is as follows:
     *  - A client emits the `awareness-update` event by sending the awareness update.
     *  - The server receives that event and applies the received update to the local awareness.
     *  - When an update is applied to awareness, the awareness "update" event will fire, which
     *    sends the update to clients connected to the document namespace.
     * @private
     * @param {Socket} socket The socket connection
     * @readonly
     */
    private readonly initAwarenessListeners;
    /**
     *  This function initializes socket event listeners for general purposes.
     * @private
     * @param {Socket} socket The socket connection
     * @readonly
     */
    private readonly initSocketListeners;
    /**
     * This function is called when a client connects and it emit the `sync-step-1` and `awareness-update`
     * events to the client to start the sync.
     * @private
     * @param {Socket} socket The socket connection
     * @param {RedisDoc} doc The document
     */
    private startSynchronization;
    /**
     * @private
     * @param {string} stream
     * @param {Array<Uint8Array>} messages
     */
    private redisMessageSubscriber;
    /**
     * @param {Namespace} namespace
     */
    getNamespaceString(namespace: Namespace): string;
    /**
     * @param {Uint8Array | ArrayBuffer | Buffer} message
     * @returns {{ type: EventType, message: Uint8Array }}
     */
    fromRedis(message: Uint8Array | ArrayBuffer | Buffer): {
        type: EventType;
        message: Uint8Array;
    };
    /**
     * @param {EventType} type
     * @param {Uint8Array | ArrayBuffer | Buffer} message
     */
    toRedis(type: EventType, message: Uint8Array | ArrayBuffer | Buffer): Uint8Array;
    destroy(): Promise<void> | undefined;
}
export type Namespace = import('socket.io').Namespace;
export type Socket = import('socket.io').Socket;
export type Server = import('socket.io').Server;
export type Subscriber = import('../subscriber.js').Subscriber;
export type UserLike = {
    userid: string;
};
export type EventType = 'sync-step-1' | 'sync-step-2' | 'sync-update' | 'awareness-update';
export type RedisDoc = {
    ydoc: Y.Doc;
    awareness: AwarenessProtocol.Awareness;
    redisLastId: string;
    storeReferences: any[] | null;
};
/**
 * YSocketIO instance configuration. Here you can configure:
 * - authenticate: The callback to authenticate the client connection
 */
export type YSocketIOConfiguration = {
    /**
     * Callback to authenticate the client connection.
     */
    authenticate: (socket: Socket) => Promise<UserLike | null> | UserLike | null;
};
import * as api from '../api.js';
import * as Y from 'yjs';
import * as AwarenessProtocol from 'y-protocols/awareness';
//# sourceMappingURL=y-socket-io.d.ts.map